// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Batch.Protocol.Models
{
    using System.Linq;

    /// <summary>
    /// Constraints to apply to the Job Manager task.
    /// </summary>
    public partial class TaskConstraints
    {
        /// <summary>
        /// Initializes a new instance of the TaskConstraints class.
        /// </summary>
        public TaskConstraints() { }

        /// <summary>
        /// Initializes a new instance of the TaskConstraints class.
        /// </summary>
        /// <param name="maxWallClockTime">The maximum elapsed time that the
        /// task may run, measured from the time the task starts. If the task
        /// does not complete within the time limit, the Batch service
        /// terminates it.</param>
        /// <param name="retentionTime">The minimum time to retain the working
        /// directory for the task on the compute node where it ran, from the
        /// time it completes execution. After this time, the Batch service
        /// may delete the working directory and all its contents.</param>
        /// <param name="maxTaskRetryCount">The maximum number of times the
        /// task may be retried. The Batch service retries a task if its exit
        /// code is nonzero.</param>
        public TaskConstraints(System.TimeSpan? maxWallClockTime = default(System.TimeSpan?), System.TimeSpan? retentionTime = default(System.TimeSpan?), int? maxTaskRetryCount = default(int?))
        {
            MaxWallClockTime = maxWallClockTime;
            RetentionTime = retentionTime;
            MaxTaskRetryCount = maxTaskRetryCount;
        }

        /// <summary>
        /// Gets or sets the maximum elapsed time that the task may run,
        /// measured from the time the task starts. If the task does not
        /// complete within the time limit, the Batch service terminates it.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "maxWallClockTime")]
        public System.TimeSpan? MaxWallClockTime { get; set; }

        /// <summary>
        /// Gets or sets the minimum time to retain the working directory for
        /// the task on the compute node where it ran, from the time it
        /// completes execution. After this time, the Batch service may
        /// delete the working directory and all its contents.
        /// </summary>
        /// <remarks>
        /// The default is infinite.
        /// </remarks>
        [Newtonsoft.Json.JsonProperty(PropertyName = "retentionTime")]
        public System.TimeSpan? RetentionTime { get; set; }

        /// <summary>
        /// Gets or sets the maximum number of times the task may be retried.
        /// The Batch service retries a task if its exit code is nonzero.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "maxTaskRetryCount")]
        public int? MaxTaskRetryCount { get; set; }

    }
}
